import numpy as np

from onnxds import datasets_pb2
from onnx import onnx_pb2

def load(filename: str):
    with open(filename, 'rb') as f:
        ds = datasets_pb2.Dataset()
        ds.ParseFromString(f.read())
        return read_ds(ds)
    return None

def read_ds(ds: datasets_pb2.Dataset):
    for it in ds.iters:
        ds_iter = read_ds_iterator(it)
        yield ds_iter

def read_ds_iterator(it: datasets_pb2.DatasetIter):
    if it.HasField('entry'):
        return read_ds_entry(it.entry)
    elif it.HasField('dicts'):
        dicts = it.dicts.dicts
        return dict([(k, read_ds_entry(dicts[k])) for k in dicts])
    else:
        raise 'no entry or dicts for DatasetIter'

def read_ds_entry(entry: datasets_pb2.DatasetEntry):
    if entry.HasField('tensor'):
        return read_ds_tensor(entry.tensor)
    elif entry.HasField('dataset'):
        return read_ds(entry.dataset)
    else:
        raise 'no tensor or dataset for DatasetEntry'

_onnx_type_match = {
    onnx_pb2.TensorProto.DataType.INT8: np.int8,
    onnx_pb2.TensorProto.DataType.UINT8: np.uint8,
    onnx_pb2.TensorProto.DataType.INT16: np.int16,
    onnx_pb2.TensorProto.DataType.UINT16: np.uint16,
    onnx_pb2.TensorProto.DataType.INT32: np.int32,
    onnx_pb2.TensorProto.DataType.UINT32: np.uint32,
    onnx_pb2.TensorProto.DataType.INT64: np.int64,
    onnx_pb2.TensorProto.DataType.UINT64: np.uint64,
    onnx_pb2.TensorProto.DataType.FLOAT16: np.single,
    onnx_pb2.TensorProto.DataType.DOUBLE: np.double,
    onnx_pb2.TensorProto.DataType.STRING: np.object
}

def read_ds_tensor(tensor: onnx_pb2.TensorProto):
    assert tensor.data_type != onnx_pb2.TensorProto.DataType.UNDEFINED and\
        tensor.data_type in _onnx_type_match,\
        'unsupported or unknown tensor data type {}'.format(tensor.data_type)
    shape = tensor.dims
    dtype = _onnx_type_match[tensor.data_type]
    raw = []
    if tensor.data_type in (
        onnx_pb2.TensorProto.DataType.INT8,
        onnx_pb2.TensorProto.DataType.UINT8,
        onnx_pb2.TensorProto.DataType.INT16,
        onnx_pb2.TensorProto.DataType.UINT16,
        onnx_pb2.TensorProto.DataType.INT32):
        raw = tensor.int32_data
    elif tensor.data_type == onnx_pb2.TensorProto.DataType.INT64:
        raw = tensor.int64_data
    elif tensor.data_type in (
        onnx_pb2.TensorProto.DataType.UINT32,
        onnx_pb2.TensorProto.DataType.UINT64):
        raw = tensor.uint64_data
    elif tensor.data_type in (
        onnx_pb2.TensorProto.DataType.FLOAT16,
        onnx_pb2.TensorProto.DataType.DOUBLE):
        raw = tensor.float_data
    elif tensor.data_type == onnx_pb2.TensorProto.DataType.STRING:
        raw = tensor.string_data
    return np.array(raw, dtype=dtype).reshape(shape)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: onnxds/datasets.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from onnx import onnx_pb2 as onnx_dot_onnx__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='onnxds/datasets.proto',
  package='onnx_dataset',
  syntax='proto3',
  serialized_options=_b('H\003'),
  serialized_pb=_b('\n\x15onnxds/datasets.proto\x12\x0connx_dataset\x1a\x0fonnx/onnx.proto\"f\n\x0c\x44\x61tasetEntry\x12#\n\x06tensor\x18\x01 \x01(\x0b\x32\x11.onnx.TensorProtoH\x00\x12(\n\x07\x64\x61taset\x18\x02 \x01(\x0b\x32\x15.onnx_dataset.DatasetH\x00\x42\x07\n\x05\x65ntry\"\x8a\x01\n\nDatasetMap\x12\x32\n\x05\x64icts\x18\x01 \x03(\x0b\x32#.onnx_dataset.DatasetMap.DictsEntry\x1aH\n\nDictsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12)\n\x05value\x18\x02 \x01(\x0b\x32\x1a.onnx_dataset.DatasetEntry:\x02\x38\x01\"m\n\x0b\x44\x61tasetIter\x12+\n\x05\x65ntry\x18\x01 \x01(\x0b\x32\x1a.onnx_dataset.DatasetEntryH\x00\x12)\n\x05\x64icts\x18\x02 \x01(\x0b\x32\x18.onnx_dataset.DatasetMapH\x00\x42\x06\n\x04iter\"3\n\x07\x44\x61taset\x12(\n\x05iters\x18\x01 \x03(\x0b\x32\x19.onnx_dataset.DatasetIterB\x02H\x03\x62\x06proto3')
  ,
  dependencies=[onnx_dot_onnx__pb2.DESCRIPTOR,])




_DATASETENTRY = _descriptor.Descriptor(
  name='DatasetEntry',
  full_name='onnx_dataset.DatasetEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='tensor', full_name='onnx_dataset.DatasetEntry.tensor', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='dataset', full_name='onnx_dataset.DatasetEntry.dataset', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='entry', full_name='onnx_dataset.DatasetEntry.entry',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=56,
  serialized_end=158,
)


_DATASETMAP_DICTSENTRY = _descriptor.Descriptor(
  name='DictsEntry',
  full_name='onnx_dataset.DatasetMap.DictsEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='key', full_name='onnx_dataset.DatasetMap.DictsEntry.key', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value', full_name='onnx_dataset.DatasetMap.DictsEntry.value', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=_b('8\001'),
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=227,
  serialized_end=299,
)

_DATASETMAP = _descriptor.Descriptor(
  name='DatasetMap',
  full_name='onnx_dataset.DatasetMap',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='dicts', full_name='onnx_dataset.DatasetMap.dicts', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_DATASETMAP_DICTSENTRY, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=161,
  serialized_end=299,
)


_DATASETITER = _descriptor.Descriptor(
  name='DatasetIter',
  full_name='onnx_dataset.DatasetIter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='entry', full_name='onnx_dataset.DatasetIter.entry', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='dicts', full_name='onnx_dataset.DatasetIter.dicts', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='iter', full_name='onnx_dataset.DatasetIter.iter',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=301,
  serialized_end=410,
)


_DATASET = _descriptor.Descriptor(
  name='Dataset',
  full_name='onnx_dataset.Dataset',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='iters', full_name='onnx_dataset.Dataset.iters', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=412,
  serialized_end=463,
)

_DATASETENTRY.fields_by_name['tensor'].message_type = onnx_dot_onnx__pb2._TENSORPROTO
_DATASETENTRY.fields_by_name['dataset'].message_type = _DATASET
_DATASETENTRY.oneofs_by_name['entry'].fields.append(
  _DATASETENTRY.fields_by_name['tensor'])
_DATASETENTRY.fields_by_name['tensor'].containing_oneof = _DATASETENTRY.oneofs_by_name['entry']
_DATASETENTRY.oneofs_by_name['entry'].fields.append(
  _DATASETENTRY.fields_by_name['dataset'])
_DATASETENTRY.fields_by_name['dataset'].containing_oneof = _DATASETENTRY.oneofs_by_name['entry']
_DATASETMAP_DICTSENTRY.fields_by_name['value'].message_type = _DATASETENTRY
_DATASETMAP_DICTSENTRY.containing_type = _DATASETMAP
_DATASETMAP.fields_by_name['dicts'].message_type = _DATASETMAP_DICTSENTRY
_DATASETITER.fields_by_name['entry'].message_type = _DATASETENTRY
_DATASETITER.fields_by_name['dicts'].message_type = _DATASETMAP
_DATASETITER.oneofs_by_name['iter'].fields.append(
  _DATASETITER.fields_by_name['entry'])
_DATASETITER.fields_by_name['entry'].containing_oneof = _DATASETITER.oneofs_by_name['iter']
_DATASETITER.oneofs_by_name['iter'].fields.append(
  _DATASETITER.fields_by_name['dicts'])
_DATASETITER.fields_by_name['dicts'].containing_oneof = _DATASETITER.oneofs_by_name['iter']
_DATASET.fields_by_name['iters'].message_type = _DATASETITER
DESCRIPTOR.message_types_by_name['DatasetEntry'] = _DATASETENTRY
DESCRIPTOR.message_types_by_name['DatasetMap'] = _DATASETMAP
DESCRIPTOR.message_types_by_name['DatasetIter'] = _DATASETITER
DESCRIPTOR.message_types_by_name['Dataset'] = _DATASET
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

DatasetEntry = _reflection.GeneratedProtocolMessageType('DatasetEntry', (_message.Message,), {
  'DESCRIPTOR' : _DATASETENTRY,
  '__module__' : 'onnxds.datasets_pb2'
  # @@protoc_insertion_point(class_scope:onnx_dataset.DatasetEntry)
  })
_sym_db.RegisterMessage(DatasetEntry)

DatasetMap = _reflection.GeneratedProtocolMessageType('DatasetMap', (_message.Message,), {

  'DictsEntry' : _reflection.GeneratedProtocolMessageType('DictsEntry', (_message.Message,), {
    'DESCRIPTOR' : _DATASETMAP_DICTSENTRY,
    '__module__' : 'onnxds.datasets_pb2'
    # @@protoc_insertion_point(class_scope:onnx_dataset.DatasetMap.DictsEntry)
    })
  ,
  'DESCRIPTOR' : _DATASETMAP,
  '__module__' : 'onnxds.datasets_pb2'
  # @@protoc_insertion_point(class_scope:onnx_dataset.DatasetMap)
  })
_sym_db.RegisterMessage(DatasetMap)
_sym_db.RegisterMessage(DatasetMap.DictsEntry)

DatasetIter = _reflection.GeneratedProtocolMessageType('DatasetIter', (_message.Message,), {
  'DESCRIPTOR' : _DATASETITER,
  '__module__' : 'onnxds.datasets_pb2'
  # @@protoc_insertion_point(class_scope:onnx_dataset.DatasetIter)
  })
_sym_db.RegisterMessage(DatasetIter)

Dataset = _reflection.GeneratedProtocolMessageType('Dataset', (_message.Message,), {
  'DESCRIPTOR' : _DATASET,
  '__module__' : 'onnxds.datasets_pb2'
  # @@protoc_insertion_point(class_scope:onnx_dataset.Dataset)
  })
_sym_db.RegisterMessage(Dataset)


DESCRIPTOR._options = None
_DATASETMAP_DICTSENTRY._options = None
# @@protoc_insertion_point(module_scope)



import unittest

import onnxds.read_dataset as helper

class ReadTest(unittest.TestCase):
    def test_single_entry(self):
        dsgen = helper.load('onnxds/mnist.onnx')
        self.assertIsNotNone(dsgen)
        it = next(dsgen)
        self.assertIsInstance(it, dict)
        self.assertIn('label', it)
        self.assertIn('image', it)
        label = it['label']
        image = it['image']
        self.assertEqual((1,), label.shape)
        self.assertEqual((1, 28, 28, 1), image.shape)

    def test_batch_entry(self):
        dsgen = helper.load('onnxds/mnist_5batch.onnx')
        self.assertIsNotNone(dsgen)
        it = next(dsgen)
        self.assertIsInstance(it, dict)
        self.assertIn('label', it)
        self.assertIn('image', it)
        label = it['label']
        image = it['image']
        self.assertEqual((5,), label.shape)
        self.assertEqual((5, 28, 28, 1), image.shape)

if __name__ == "__main__":
    unittest.main()



from setuptools import setup, find_packages

__version__ = "0.0.2"

def readme():
    with open("README.md") as f:
        return f.read()

setup(
    name = "onnxds",
    version = __version__,
    description = "Create datasets using onnx and serialize tensorflow_datasets in onnx datasets.",
    long_description = readme(),
    long_description_content_type ="text/markdown",
    classifiers = [
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: POSIX",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
    ],
    keywords = "",
    url = "https://github.com/mingkaic/onnxds",
    author = "Ming Kai Chen",
    author_email = "mingkaichen2009@gmail.com",
    license = "MIT",
    packages = find_packages(),
    install_requires = ['tensorflow', 'tensorflow_datasets'],
    test_suite = "",
    tests_require = [],
    zip_safe = False,
    python_requires = '>=3.6',
)



import argparse
import sys

import tensorflow as tf
import tensorflow_datasets as tfds
import numpy as np

from onnxds import datasets_pb2
from onnx import onnx_pb2

def tfwrite_ds(ox: datasets_pb2.Dataset, ds: tf.data.Dataset):
    for el in ds:
        tfwrite_ds_iterator(ox.iters.add(), el)

def tfwrite_ds_iterator(ox: datasets_pb2.DatasetIter, el: any):
    if isinstance(el, dict):
        for key in el:
            tfwrite_ds_entry(ox.dicts.dicts[key], el[key])
    else:
        tfwrite_ds_entry(ox.entry, el)

def tfwrite_ds_entry(ox: datasets_pb2.DatasetEntry, el: any):
    if isinstance(el, tf.Tensor):
        tfwrite_tensor(ox.tensor, el)
    elif isinstance(el, tf.data.Dataset):
        tfwrite_ds(ox.dataset, el)
    else:
        assert('unhandled type {}'.format(type(el)))

def tfwrite_tensor(ox: onnx_pb2.TensorProto, tens: tf.Tensor):
    data = tens.numpy()
    shape = data.shape

    if data.dtype == np.int8:
        dtype = onnx_pb2.TensorProto.DataType.INT8
    elif data.dtype == np.uint8:
        dtype = onnx_pb2.TensorProto.DataType.UINT8
    elif data.dtype == np.int16:
        dtype = onnx_pb2.TensorProto.DataType.INT16
    elif data.dtype == np.uint16:
        dtype = onnx_pb2.TensorProto.DataType.UINT16
    elif data.dtype == np.int32:
        dtype = onnx_pb2.TensorProto.DataType.INT32
    elif data.dtype == np.uint32:
        dtype = onnx_pb2.TensorProto.DataType.UINT32
    elif data.dtype == np.int64:
        dtype = onnx_pb2.TensorProto.DataType.INT64
    elif data.dtype == np.uint64:
        dtype = onnx_pb2.TensorProto.DataType.UINT64
    elif data.dtype == np.single:
        dtype = onnx_pb2.TensorProto.DataType.FLOAT16
    elif data.dtype == np.double:
        dtype = onnx_pb2.TensorProto.DataType.DOUBLE
    else:
        dtype = onnx_pb2.TensorProto.DataType.UNDEFINED
    ox.dims.extend(shape)
    ox.data_type = dtype
    if dtype in (
        onnx_pb2.TensorProto.DataType.INT8,
        onnx_pb2.TensorProto.DataType.UINT8,
        onnx_pb2.TensorProto.DataType.INT16,
        onnx_pb2.TensorProto.DataType.UINT16,
        onnx_pb2.TensorProto.DataType.INT32):
        ox.int32_data.extend(data.flatten())
    elif dtype == onnx_pb2.TensorProto.DataType.INT64:
        ox.int64_data.extend(data.flatten())
    elif dtype in (
        onnx_pb2.TensorProto.DataType.UINT32,
        onnx_pb2.TensorProto.DataType.UINT64):
        ox.uint64_data.extend(data.flatten())
    elif dtype in (
        onnx_pb2.TensorProto.DataType.FLOAT16,
        onnx_pb2.TensorProto.DataType.DOUBLE):
        ox.float_data.extend(data.flatten())
    else:
        raw = data.flatten()
        if isinstance(raw[0], bytes) or isinstance(raw[0], str):
            ox.data_type = onnx_pb2.TensorProto.DataType.STRING
            ox.string_data.extend(raw)
        else:
            assert False, 'failed to serialize unknown datatype {}: {}...'.format(type(raw[0]), raw[0])

def tfgen_model(name: str, fname = None, **kwargs):
    ox_ds = datasets_pb2.Dataset()
    tf_ds = tfds.load(name, **kwargs)
    tfwrite_ds(ox_ds, tf_ds)
    if fname is None:
        fname = '/tmp/{}.onnx'.format(name)
    with open(fname, "wb") as f:
        f.write(ox_ds.SerializeToString())

if __name__ == '__main__':
    prog_description = 'Convert tensorflow dataset to onnx dataset'
    parser = argparse.ArgumentParser(description=prog_description)
    parser.add_argument('name')
    parser.add_argument('--out', dest='out',
        type=str, nargs='?', default=None,
        help='Filename to generate onnx dataset')
    parser.add_argument('--split', dest='split',
        type=str, nargs='?', default=None,
        help='Tfds.Split subset: one of TRAIN, VALIDATION, TEST, or any other option uses default')
    parser.add_argument('--batch', dest='batch',
        type=int, nargs='?', default=1,
        help='Tfds batch argument')
    args = parser.parse_args(sys.argv[1:])

    argvs = {
        'batch_size': args.batch
    }
    if args.split == 'TRAIN':
        argvs['split'] = tfds.Split.TRAIN
    elif args.split == 'VALIDATION':
        argvs['split'] = tfds.Split.VALIDATION
    elif args.split == 'TEST':
        argvs['split'] = tfds.Split.TEST
    tfgen_model(args.name, fname=args.out, **argvs)
